import 'package:chessjs/chessjs.dart' as chessjs;
import 'package:chessjs/color.dart' as chessColor;
import 'constants.dart';
import 'package:flutter/material.dart';

typedef Null MoveCallback(String moveNotation);
typedef Null CheckMateCallback(ChessboardPieceColor color);
typedef Null CheckCallback(ChessboardPieceColor color);
typedef Null DrawCallback();

class ChessboardController extends ChangeNotifier {
  ChessboardController(
    this.onMove,
    this.onCheckMate,
    this.onCheck,
    this.onDraw,
    this.whiteSideTowardsUser,
    this.enableUserMoves,
    chessjs.Chess? startingGame,
  ) {
    if (startingGame == null) {
      gameLogic = chessjs.Chess();
    } else
      gameLogic = startingGame;
  }

  /// Creates a logical chess game
  late chessjs.Chess gameLogic;

  /// Callback for when a move is made
  MoveCallback onMove;

  /// Callback for when a player is checkmated
  CheckMateCallback onCheckMate;

  ///Callback for when a player is in check
  CheckCallback onCheck;

  /// Callback for when the game is a draw (Example: K v K)
  DrawCallback onDraw;

  /// If the white side of the board is towards the user
  bool whiteSideTowardsUser;

  /// User moves can be enabled or disabled by this property
  bool enableUserMoves;

  /// Refreshes board
  void refreshBoard() {
    if (gameLogic.inCheckmate) {
      onCheckMate(gameLogic.turn == chessColor.Color.WHITE
          ? ChessboardPieceColor.White
          : ChessboardPieceColor.Black);
    } else if (gameLogic.inDraw ||
        gameLogic.inStalemate ||
        gameLogic.inThreefoldRepetition ||
        gameLogic.insufficientMaterial) {
      onDraw();
    } else if (gameLogic.inCheck) {
      onCheck(gameLogic.turn == chessColor.Color.WHITE
          ? ChessboardPieceColor.White
          : ChessboardPieceColor.Black);
    }
  }

  void onPromotion() {
    notifyListeners();
  }
}
